@@ -1,14 +1,17 @@
-from typing import Any, Dict, List, Optional, Tuple, TypeVar, Union, cast, overload
+from typing import Any, Dict, List, Optional, Tuple, TypeVar, Union, cast, overload, Iterable, TypeVar
 import httpx
 import logging
 import json
+import itertools
 from urllib.parse import quote
 from secrets import token_bytes, token_hex
 from base64 import b64decode, b64encode
+from functools import wraps
+
 
 from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
 from cryptography.hazmat.primitives.padding import PKCS7
-from cryptography.hazmat.primitives.hashes import SHA256
+from cryptography.hazmat.primitives.hashes import SHA1
 from cryptography.hazmat.primitives.hmac import HMAC
 
 from .typing import (
@@ -23,8 +26,10 @@ from .typing import (
     ChannelListType,
     ChannelsCurrentType,
     ChannelsType,
+    ChannelType,
     ContextType,
     FavoriteListType,
+    RecordingsListed,
     MenuItemsSettingsCurrent,
     MenuItemsSettingsCurrentPost,
     MenuItemsSettingsCurrentValueValue,
@@ -33,11 +38,12 @@ from .typing import (
     MenuItemsSettingsStructure,
     MenuItemsSettingsUpdate,
     MenuItemsSettingsValueData,
+    Strings,
+    StringsRequest,
     SystemType,
     ApplicationType,
 )
 from .auth import CachedDigestAuth
-from httpcore import AsyncConnectionPool
 
 LOG = logging.getLogger(__name__)
 TIMEOUT = 20.0
@@ -61,11 +67,12 @@ TV_PLAYBACK_INTENTS = [
 
 HTTP_PORT = 1925
 HTTPS_PORT = 1926
-
+MAXIMUM_ITEMS_IN_REQUEST = 50
+IGNORED_JSON_RESPONSES = {"", "Context Service not started", "}"}
 
 def hmac_signature(key: bytes, timestamp: str, data: str):
     """Calculate a timestamped signature."""
-    hmac = HMAC(key, SHA256())
+    hmac = HMAC(key, SHA1())
     hmac.update(timestamp.encode("utf-8"))
     hmac.update(data.encode("utf-8"))
     return b64encode(hmac.finalize()).decode("utf-8")
@@ -96,7 +103,7 @@ def cbc_encode(key: bytes, data: str):
 
 
 def decode_xtv_json(text: str):
-    if text == "" or text == "}":
+    if text in IGNORED_JSON_RESPONSES:
         LOG.debug("Ignoring invalid json %s", text)
         return {}
 
@@ -108,7 +115,11 @@ def decode_xtv_json(text: str):
         text = text.replace(",}", "}")
         while (p := text.find(",,")) >= 0:
             text = text[:p] + text[p + 1 :]
-        data = json.loads(text)
+
+        try:
+            data = json.loads(text)
+        except json.decoder.JSONDecodeError as exception:
+            raise NoneJsonData(text) from exception
 
     return data
 
@@ -128,6 +139,20 @@ def decode_xtv_response(response: httpx.Response):
     return decode_xtv_json(text)
 
 
+_T = TypeVar("_T")
+
+def chunked_iterator(size: int, iterable: Iterable[_T]) -> Iterable[Iterable[_T]]:
+    """Create a chain of iterators that each will give a chunk of the original."""
+    step = iter(iterable)
+    while True:
+        chunk_it = itertools.islice(step, size)
+        try:
+            first_el = next(chunk_it)
+        except StopIteration:
+            return
+        yield itertools.chain((first_el,), chunk_it)
+
+
 PASSTHROUGH_URI = "content://android.media.tv/passthrough"
 
 
@@ -168,58 +193,41 @@ class PairingFailure(GeneralFailure):
 
 
 class NoneJsonData(GeneralFailure):
+    """API Returned non json data when json was expected."""
     def __init__(self, data):
         super().__init__(f"Non json data received: {data}")
         self.data = data
 
-    """API Returned non json data when json was expected."""
-
 
 T = TypeVar("T")
 
-def track_removed_connections(session: httpx.AsyncClient):
-    """
-    httpcore version up to at least 1.13.7 series contain a bug where a session is leaked
-    if the server closes the connection, a pull request to solve the issue exists here
-    https://github.com/encode/httpcore/pull/117 but so far it's not been merged.
 
-    Version 1.14.x of httpx library is re-designed and is not yet known to suffer this bug,
-    it however might suffer another similar: https://github.com/encode/httpcore/issues/431
-    """
-    removed_connections = set()
-    
-    try:
-        pool: AsyncConnectionPool = session._transport._pools
-        original_add_to_pool = pool._add_to_pool
-        original_remove_from_pool = pool._remove_from_pool
-    except AttributeError:
-        LOG.debug("Object to patch is missing, maybe library have been upgraded so skipping")
-        return removed_connections
+def handle_httpx_exceptions(f):
+    """Wrap up httpx exceptions in our wanted variants."""
+    @wraps(f)
+    async def wrapper(*args, **kwds):
+        try:
+            try:
+                return await f(*args, **kwds)
+            except httpx.RemoteProtocolError as err:
+                LOG.debug("%r. We retry once, could be a reused session that was closed", err)
+                return await f(*args, **kwds)
 
-    async def _add_to_pool(
-        connection, timeout
-    ) -> None:
-        if connection in removed_connections:
-            removed_connections.remove(connection)
-        await original_add_to_pool(connection, timeout)
-    pool._add_to_pool = _add_to_pool
-    async def _remove_from_pool(
-        connection
-    ) -> None:
-        removed_connections.add(connection)
-        await original_remove_from_pool(connection)
-    pool._remove_from_pool = _remove_from_pool
-    return removed_connections
+        except (httpx.ConnectTimeout, httpx.ConnectError) as err:
+            raise ConnectionFailure(err) from err
+        except (httpx.ProtocolError, httpx.ReadError) as err:
+            raise ProtocolFailure(err) from err
+        except httpx.HTTPError as err:
+            raise GeneralFailure(err) from err
 
+    return wrapper
 
-async def close_removed_connections(removed_connections: set):
-    for connection in removed_connections:
-        LOG.warning("Closing connection leaked from pool")
-        if connection.should_close():
-            await connection.aclose()
-    removed_connections.clear()
 
 class PhilipsTV(object):
+
+    channels: ChannelsType
+    """All available channels, with ccid as key."""
+
     def __init__(
         self,
         host=None,
@@ -229,7 +237,8 @@ class PhilipsTV(object):
         password=None,
         verify=False,
         auth_shared_key=None,
-        system = None
+        system = None,
+        limits = None,
     ):
         self._host = host
         self._connfail = 0
@@ -237,10 +246,11 @@ class PhilipsTV(object):
         self.on = False
         self.name: Optional[str] = None
         self.system: Optional[SystemType] = system
+        self.strings: dict[str, str] = {}
         self.sources = {}
         self.source_id = None
         self.audio_volume = None
-        self.channels: ChannelsType = {}
+        self.channels = {}
         self.channel: Optional[Union[ActivitesTVType, ChannelsCurrentType]] = None
         self.channel_lists: Dict[str, ChannelListType] = {}
         self.favorite_lists: Dict[str, FavoriteListType] = {}
@@ -261,6 +271,7 @@ class PhilipsTV(object):
         self.ambilight_current_configuration: Optional[
             AmbilightCurrentConfiguration
         ] = None
+        self.recordings_list: Optional[RecordingsListed] = None
         self.huelamp_power: Optional[str] = None
         self.powerstate = None
         if auth_shared_key:
@@ -274,12 +285,11 @@ class PhilipsTV(object):
             self.protocol = "http"
 
         timeout = httpx.Timeout(timeout=TIMEOUT, connect=TIMEOUT_CONNECT)
-        limits = httpx.Limits(max_keepalive_connections=0, max_connections=3)
+        if limits is None:
+            limits = httpx.Limits(max_keepalive_connections=3, max_connections=3)
         self.session = httpx.AsyncClient(limits=limits, timeout=timeout, verify=False)
         self.session.headers["Accept"] = "application/json"
 
-        self.removed_connections = track_removed_connections(self.session)
-
         if username and password:
             self.session.auth = CachedDigestAuth(username, password)
 
@@ -364,7 +374,7 @@ class PhilipsTV(object):
                 self.system.get("featuring", {})
                 .get("systemfeatures", {})
                 .get("secured_transport")
-                == "true"
+                in ("true", True)
             )
         else:
             return None
@@ -479,6 +489,44 @@ class PhilipsTV(object):
                     return r["id"][pos + 1 :]
             return r["id"]
 
+    @property
+    def channel_list_id(self) -> str:
+        if self.api_version >= 5:
+            if not self.channel:
+                return "all"
+            r = cast(ActivitesTVType, self.channel)
+            channel_list = r.get("channelList")
+            if not channel_list:
+                return "all"
+            return channel_list.get("id", "all")
+
+        return "all"
+
+    @property
+    def channels_current(self) -> List[ChannelType]:
+        """All channels in the current favorite list."""
+        if self.api_version >= 5:
+            favorite_list = self.favorite_lists.get(self.channel_list_id)
+            if not favorite_list:
+                return list(self.channels.values())
+
+            return [
+                {
+                    **channel,
+                    "preset": favorite.get("preset", "")
+                }
+                for favorite in favorite_list.get("channels", [])
+                if (channel := self.channels.get(str(favorite.get("ccid"))))
+            ]
+        else:
+            return [
+                {
+                    **channel,
+                    "ccid": key
+                }
+                for key, channel in self.channels.items()
+            ]
+
     @property
     def ambilight_modes(self):
         modes = ["internal", "manual", "expert"]
@@ -537,51 +585,40 @@ class PhilipsTV(object):
         if protocol is None:
             protocol = self.protocol
 
-        if self.protocol == "https":
+        if protocol == "https":
             port = HTTPS_PORT
         else:
             port = HTTP_PORT
 
         return f"{protocol}://{self._host}:{port}/{self.api_version}/{path}"
 
-    async def getReq(self, path) -> Optional[Dict]:
+    @handle_httpx_exceptions
+    async def getReq(self, path, protocol = None) -> Optional[Dict]:
+        resp = await self.session.get(self._url(path, protocol = protocol))
 
-        try:
-            resp = await self.session.get(self._url(path))
-            if resp.status_code == 401:
-                raise AutenticationFailure("Authenticaion failed to device")
+        if resp.status_code == 401:
+            raise AutenticationFailure("Authenticaion failed to device")
 
-            if resp.status_code != 200:
-                LOG.debug("Get failed: %s -> %d %s", path, resp.status_code, resp.text)
-                return None
-
-            LOG.debug("Get succeded: %s -> %s", path, resp.text)
-            return decode_xtv_response(resp)
-        except (httpx.ConnectTimeout, httpx.ConnectError) as err:
-            raise ConnectionFailure(err) from err
-        except httpx.HTTPError as err:
-            raise GeneralFailure(err) from err
-        finally:
-            await close_removed_connections(self.removed_connections)
+        if resp.status_code != 200:
+            LOG.debug("Get failed: %s -> %d %s", path, resp.status_code, resp.text)
+            return None
 
+        LOG.debug("Get succeded: %s -> %s", path, resp.text)
+        return decode_xtv_response(resp)
+  
+    @handle_httpx_exceptions
     async def _getBinary(self, path: str) -> Tuple[Optional[bytes], Optional[str]]:
 
-        try:
-            resp = await self.session.get(self._url(path))
-            if resp.status_code == 401:
-                raise AutenticationFailure("Authenticaion failed to device")
+        resp = await self.session.get(self._url(path))
+        if resp.status_code == 401:
+            raise AutenticationFailure("Authenticaion failed to device")
 
-            if resp.status_code != 200:
-                return None, None
-            return resp.content, resp.headers.get("content-type")
-        except (httpx.ConnectTimeout, httpx.ConnectError) as err:
-            raise ConnectionFailure(err) from err
-        except httpx.HTTPError as err:
-            raise GeneralFailure(err) from err
-        finally:
-            await close_removed_connections(self.removed_connections)
+        if resp.status_code != 200:
+            return None, None
+        return resp.content, resp.headers.get("content-type")
 
-    async def postReq(self, path: str, data: Dict, timeout=None, protocol=None) -> Optional[Dict]:
+    @handle_httpx_exceptions
+    async def postReq(self, path: str, data: Any, timeout=None, protocol=None) -> Optional[Dict]:
         try:
             resp = await self.session.post(self._url(path, protocol), json=data, timeout=timeout)
             if resp.status_code == 401:
@@ -596,14 +633,6 @@ class PhilipsTV(object):
         except httpx.ReadTimeout:
             LOG.debug("Read time out on postReq", exc_info=True)
             return None
-        except (httpx.ConnectTimeout, httpx.ConnectError) as err:
-            raise ConnectionFailure(err) from err
-        except (httpx.ProtocolError, httpx.ReadError) as err:
-            raise ProtocolFailure(err) from err
-        except httpx.HTTPError as err:
-            raise GeneralFailure(err) from err
-        finally:
-            await close_removed_connections(self.removed_connections)
 
     async def pairRequest(
         self,
@@ -629,7 +658,18 @@ class PhilipsTV(object):
 
         state = {"device": device}
 
-        data = {"scope": ["read", "write", "control"], "device": device}
+        data = {
+            "access": {
+                "scope": ["read", "write", "control"]
+            },
+            "device": device
+        }
+
+        if self.system:
+            featuring = self.system.get("featuring", None)
+            if featuring:
+                data["access"]["featuring"] = featuring
+
 
         LOG.debug("pair/request request: %s", data)
         resp = await self.session.post(self._url("pair/request"), json=data, auth=None)
@@ -716,6 +756,7 @@ class PhilipsTV(object):
             await self.getAmbilightPower()
             await self.getAmbilightCurrentConfiguration()
             await self.getHueLampPower()
+            await self.getRecordings()
             self.on = True
             return True
         except ConnectionFailure as err:
@@ -733,13 +774,21 @@ class PhilipsTV(object):
         return cast(SystemType, result)
 
     async def getSystem(self):
-        r = cast(Optional[SystemType], await self.getReq("system"))
-        if r:
-            self.system = self._decodeSystem(r)
-            self.name = r.get("name")
-        else:
-            self.system = {}
-            self.name = None
+        # Newest TV software requires https for system-info. Therefore we will try both protocols.
+        protocols = ["http", "https"]
+
+        if protocols[0] != self.protocol:
+            protocols = reversed(protocols)
+        
+        for prot in protocols:
+            r = cast(Optional[SystemType], await self.getReq("system", protocol=prot))
+            if r:
+                self.system = self._decodeSystem(r)
+                self.name = r.get("name")
+                return r
+            else:
+                self.system = {}
+                self.name = None
         return r
 
     async def getAudiodata(self):
@@ -753,13 +802,10 @@ class PhilipsTV(object):
     async def getChannels(self):
         if self.api_version >= 5:
             self.channels = {}
-            for list_id in self.channel_lists:
-                r = await self.getChannelList(list_id)
-                if r:
-                    for channel in r:
-                        if "ccid" in channel:
-                            self.channels[str(channel["ccid"])] = channel
-                return r
+            for channel_list in self.channel_lists.values():
+                for channel in channel_list.get("Channel", []):
+                    self.channels[str(channel.get("ccid"))] = channel
+            return self.channels
         else:
             r = cast(Optional[ChannelsType], await self.getReq("channels"))
             if r:
@@ -804,15 +850,15 @@ class PhilipsTV(object):
         self.context = r
         return r
 
-    async def setChannel(self, ccid, list_id: str = "alltv"):
+    async def setChannel(self, ccid: Union[str, int], list_id: Optional[str] = None):
         channel: Union[ActivitesTVType, ChannelsCurrentType]
         if self.api_version >= 5:
-            channel = {"channelList": {"id": list_id}, "channel": {"ccid": ccid}}
+            channel = {"channelList": {"id": list_id or "all"}, "channel": {"ccid": int(ccid)}}
             if await self.postReq("activities/tv", cast(Dict, channel)) is not None:
                 self.channel = channel
                 return True
         else:
-            channel = {"id": ccid}
+            channel = {"id": str(ccid)}
             if await self.postReq("channels/current", cast(Dict, channel)) is not None:
                 self.channel = channel
                 return True
@@ -822,15 +868,34 @@ class PhilipsTV(object):
         if self.api_version >= 5:
             r = cast(ChannelDbTv, await self.getReq("channeldb/tv"))
             if r:
-                self.channel_lists = {
-                    data["id"]: data for data in r.get("channelLists", {})
-                }
-                self.favorite_lists = {
-                    data["id"]: data for data in r.get("favoriteLists", {})
-                }
+                channel_lists = {}
+                favorite_lists = {}
+
+                for data in r.get("channelLists", []):
+                    list_id = data["id"]
+                    channel_list = cast(
+                        Optional[ChannelListType],
+                        await self.getReq(f"channeldb/tv/channelLists/{list_id}"),
+                    )
+                    if channel_list:
+                        channel_lists[list_id] = channel_list
+
+
+                for data in r.get("favoriteLists", []):
+                    list_id = data["id"]
+                    favorite_list = cast(
+                        Optional[FavoriteListType],
+                        await self.getReq(f"channeldb/tv/favoriteLists/{list_id}"),
+                    )
+                    if favorite_list:
+                        favorite_lists[list_id] = favorite_list
+
+                self.channel_lists = channel_lists
+                self.favorite_lists = favorite_lists
             else:
                 self.channel_lists = {}
                 self.favorite_lists = {}
+
             return r
 
     async def getFavoriteList(self, list_id: str):
@@ -1256,19 +1321,64 @@ class PhilipsTV(object):
 
             return True
 
+    async def getRecordings(self):
+        #Just known working with API level 6 currently
+        if self.api_version == 6:
+            if self.json_feature_supported("recordings", "List"):
+                r = cast(
+                    Optional[RecordingsListed],
+                    await self.getReq("recordings/list"),
+                )
+                if r:
+                    self.recordings_list = r
+                else:
+                    self.recordings_list = None
+                return r
+
     async def openURL(self, url: str):
         if self.json_feature_supported("activities", "browser"):
             r = await self.postReq("activities/browser", {"url": url})
             return r is not None
 
+    async def getStringsCached(self, string_ids: Iterable[str]) -> Union[Dict[str, str], None]:
+        """Fetch translations for strings, making sure all items exist."""
+        string_ids_wanted = set(string_ids)
+        string_ids_missing = string_ids_wanted - self.strings.keys()
+
+        # fetch translations from TV
+        if string_ids_missing:
+            if (data := await self.getStrings(sorted(string_ids_missing))) is None:
+                return None
+            self.strings.update(data)
+
+        # make sure untranslated strings is set 1 to 1
+        string_ids_missing = string_ids_wanted - set(self.strings.keys())
+        for string_id in string_ids_missing:
+            self.strings[string_id] = string_id
+
+        return self.strings
+
     async def getStrings(
         self,
-        strings: List[str],
+        strings: Iterable[str],
         language: Optional[str] = None,
         country: Optional[str] = None,
         variant: Optional[str] = None,
-    ):
-        data = {
+    ) -> Dict[str, str]:
+        res: Dict[str, str] = {}
+        for group in chunked_iterator(MAXIMUM_ITEMS_IN_REQUEST, strings):
+            data = await self._getStrings(group, language=language, country=country, variant=variant)
+            res.update(data)
+        return res
+
+    async def _getStrings(
+        self,
+        strings: Iterable[str],
+        language: Optional[str] = None,
+        country: Optional[str] = None,
+        variant: Optional[str] = None,
+    ) -> Dict[str, str]:
+        data: StringsRequest = {
             "strings": [{"string_id": string} for string in strings],
             "locale": {
                 "language": language or "",
@@ -1276,14 +1386,14 @@ class PhilipsTV(object):
                 "variant": variant or "",
             },
         }
-        res = await self.postReq("strings", data)
+        res = cast(Optional[Strings], await self.postReq("strings", data))
         if res:
             return {
                 translation["string_id"]: translation["string_translation"]
                 for translation in res["translations"]
                 if translation["string_translation"]
             }
-        return None
+        return {}
 
     async def getMenuItemsSettingsStructure(self, force=False) -> Optional[MenuItemsSettingsStructure]:
         if self.json_feature_supported("menuitems", "Setup_Menu") or force:
@@ -1293,8 +1403,18 @@ class PhilipsTV(object):
             self.settings = r
             return r
 
+    async def getMenuItemsSettingsCurrent(self, node_ids: Iterable[int], force=False) -> Optional[MenuItemsSettingsCurrent]:
+        current: MenuItemsSettingsCurrent = {"values": [], "version": 0}
+        for group in chunked_iterator(MAXIMUM_ITEMS_IN_REQUEST, node_ids):
+            if (data := await self._getMenuItemsSettingsCurrent(group)) is None:
+                return None
 
-    async def getMenuItemsSettingsCurrent(self, node_ids: List[int], force=False) -> Optional[MenuItemsSettingsCurrent]:
+            current["values"].extend(data["values"])
+            if current["version"] == 0:
+                current["version"] = data["version"]
+        return current
+
+    async def _getMenuItemsSettingsCurrent(self, node_ids: Iterable[int], force=False) -> Optional[MenuItemsSettingsCurrent]:
         if not node_ids:
             return None
 
