@@ -15,7 +15,7 @@ class ActivitiesChannelType(TypedDict, total=False):
 
 class ActivitiesChannelListType(TypedDict, total=False):
     id: str
-    version: str
+    version: Union[int, str]
 
 
 class ActivitesTVType(TypedDict, total=False):
@@ -51,7 +51,7 @@ class ApplicationsType(TypedDict):
     applications: List[ApplicationType]
 
 
-class FavoriteChannelType(TypedDict):
+class FavoriteChannelType(TypedDict, total=False):
     ccid: int
     preset: str
 
@@ -59,19 +59,22 @@ class FavoriteChannelType(TypedDict):
 class FavoriteListType(TypedDict, total=False):
     id: str
     version: Union[int, str]
-    parentId: str
     listType: str
     medium: str
-    virtual: bool
-    modifiable: bool
     name: str
     channels: List[FavoriteChannelType]
 
 
 class ChannelType(TypedDict, total=False):
-    ccid: int
+    ccid: Union[int, str]
     preset: str
     name: str
+    onid: int
+    tsid: int
+    sid: int
+    serviceType: str
+    type: str
+    logoVersion: Union[int, str]
 
 
 class ChannelListType(TypedDict, total=False):
@@ -79,18 +82,37 @@ class ChannelListType(TypedDict, total=False):
     version: Union[int, str]
     listType: str
     medium: str
-    active: bool
-    virtual: bool
-    modifiable: bool
+    operator: str
+    installCountry: str
     Channel: List[ChannelType]
 
 
 ChannelsType = Dict[str, ChannelType]
 
 
+class ChannelDbTvListBase(TypedDict):
+    id: str
+    version: Union[int, str]
+    listType: str
+
+
+class ChannelDbTvList(ChannelDbTvListBase, total=False):
+    medium: str
+    active: bool
+    virtual: bool
+    modifiable: bool
+
+class ChannelDbTvListFavorite(ChannelDbTvListBase, total=False):
+    medium: str
+    active: bool
+    virtual: bool
+    modifiable: bool
+    parentId: str
+    name: str
+
 class ChannelDbTv(TypedDict, total=False):
-    channelLists: List[ChannelListType]
-    favoriteLists: List[FavoriteListType]
+    channelLists: List[ChannelDbTvList]
+    favoriteLists: List[ChannelDbTvListFavorite]
 
 
 class JsonFeaturesType(TypedDict, total=False):
@@ -220,6 +242,50 @@ class AmbilightCurrentConfiguration(TypedDict, total=False):
     audioSettings: AmbilightAudioSettingsType
     colorSettings: AmbilightColorSettingsType
 
+class RecordingsListedType(TypedDict, total=False):
+    RecordingId: int
+    ServerRecordingId: int
+    RecordingType: str #RECORDING_ONGOING, RECORDING_SCHEDULED, RECORDING_NEW, RECORDING_PARTIALLY_VIEWED, RECORDING_VIEWED
+    RecName: str
+    EventId: int
+    EventInfo: str
+    EventExtendedInfo: str
+    EventGenre: str
+    EventRating: int
+    SeriesID: str
+    StartTime: int
+    ActualStartTime: int
+    Duration: int
+    MarginStart: int
+    MarginEnd: int
+    AutoMarginStart: int
+    AutoMarginEnd: int
+    RetentionInfo: int
+    SeasonNo: int
+    EpisodeNo: int
+    EpisodeCount: int
+    ProgramNumber: int
+    ProgramDuration: int
+    ResumeInfo: int
+    LastPinChangedTime: int
+    HasCicamPin: bool
+    hasDot: bool
+    HasLicenseFile: bool
+    IsRadio: bool
+    IsPartial: bool
+    IsIpEpgRec: bool
+    isFTARecording: bool
+    EITVersion: int
+    EITSource: str #EIT_SOURCE_PF, ...
+    RecError: str #REC_ERROR_NONE, ...
+    Version: int
+    Size: int
+    ccid: int
+
+class RecordingsListed(TypedDict):
+    version: str
+    recordings: List[RecordingsListedType]
+
 class MenuItemsSettingsNodeDataSliderData(TypedDict):
     min: int
     max: int
@@ -325,3 +391,22 @@ class MenuItemsSettingsUpdateValue(TypedDict):
 
 class MenuItemsSettingsUpdate(TypedDict):
     values: List[MenuItemsSettingsUpdateValue]
+
+class StringsTranslation(TypedDict):
+    string_id: str
+    string_translation: Optional[str]
+
+class Strings(TypedDict):
+    translations: List[StringsTranslation]
+
+class StringId(TypedDict):
+    string_id: str
+
+class StringLocale(TypedDict):
+    language: str
+    country: str
+    variant: str
+
+class StringsRequest(TypedDict):
+    strings: List[StringId]
+    locale: StringLocale
